unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TMouseState = (msSelecting, msMoving, msNone);

  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure ButtonMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure ButtonMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure ButtonMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  private
    MouseState : TMouseState;
    ptMove, ptStart : TPoint;
    procedure Draw(Sender : TWinControl);
    procedure DrawRectangle(X,Y, Width, Height : Integer);
  public
    ControlSelected : TList;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
  ControlSelected := TList.Create;
  MouseState := msNone;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  ControlSelected.Free;
end;

procedure TForm1.FormPaint(Sender: TObject);
var
  i : Integer;
begin
  // Dessine les petits carrs autour des contrles slectionns uniquement lors de la slection
  if MouseState = msSelecting then begin
    for i := 0 to ControlSelected.Count-1 do
      Draw(TWinControl(ControlSelected.Items[i]));
  end;
end;

procedure TForm1.FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  // Indique qu'on ne dplace rien ni ne slectionne
  MouseState := msNone;

  // Efface la liste des objects slectionns
  ControlSelected.Clear;

  // Rafraichit l'affichage
  Self.Refresh;
end;

procedure TForm1.FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if MouseState = msMoving then
    MouseState := msNone;
end;

procedure TForm1.ButtonMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Index : Integer;
begin
  // Multi-slection des contrles avec la touche CTRL
  if ssCtrl in Shift then begin
    MouseState := msSelecting;
    Index := ControlSelected.IndexOf(Sender);
    if Index=-1 then ControlSelected.Add(Sender)
                else ControlSelected.Delete(Index);
  end;

  // On n'appuie pas sur la touche CTRL : On s'apprte  dplacer
  if Shift = [ssLeft] then begin
    if MouseState = msSelecting then MouseState := msMoving;
    ptStart := Point(X,Y);
  end;

  // Dessine les carrs sur les contrles slectionns
  Self.Refresh;
end;

procedure TForm1.ButtonMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  i, XX, YY : Integer;
  C : TWinControl;
begin
  if MouseState = msMoving then begin
    MouseState := msSelecting;

    // Dplace les contrles
    XX := ptMove.X-ptStart.X;
    YY := ptMove.Y-ptStart.Y;
    for i := 0 to ControlSelected.Count-1 do begin
      C := TWinControl(ControlSelected.Items[i]);
      C.Left := C.Left+XX;
      C.Top := C.Top+YY;
    end;

    // Permet de redessiner les petits carrs de slection
    Self.Refresh;
  end;
end;

procedure TForm1.ButtonMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  i : Integer;
  pt : TPoint;
  C : TWinControl;
begin
  if MouseState = msMoving then begin
    ptMove := Point(X,Y); // Pour le dplacement sur la feuille
    pt.X := X-ptStart.X;
    pt.Y := Y-ptStart.Y;

    // Dssine les rectangles lors du dplacement
    Self.Refresh;
    for i := 0 to ControlSelected.Count-1 do begin
      C := TWinControl(ControlSelected.Items[i]);
      DrawRectangle(C.Left+pt.X, C.Top+pt.Y, C.Width, C.Height);
    end;
  end;
end;

// Dessine les petits carrs autour d'un contrle
procedure TForm1.Draw(Sender: TWinControl);
const
  cstWidth = 2;
var
  X, Y, XX, YY, X1, Y1: Integer;
begin
  X := Sender.Left;
  Y := Sender.Top;
  X1 := X+(Sender.Width div 2);
  Y1 := Y+(Sender.Height div 2);
  XX := X+Sender.Width;
  YY := Y+Sender.Height;

  Self.Canvas.Rectangle(X-cstWidth, Y-cstWidth, X+cstWidth, Y+cstWidth);
  Self.Canvas.Rectangle(XX-cstWidth, Y-cstWidth, XX+cstWidth, Y+cstWidth);

  Self.Canvas.Rectangle(X1-cstWidth, Y-cstWidth, X1+cstWidth, Y+cstWidth);
  Self.Canvas.Rectangle(X1-cstWidth, YY-cstWidth, X1+cstWidth, YY+cstWidth);

  Self.Canvas.Rectangle(X-cstWidth, YY-cstWidth, X+cstWidth, YY+cstWidth);
  Self.Canvas.Rectangle(XX-cstWidth, YY-cstWidth, XX+cstWidth, YY+cstWidth);

  Self.Canvas.Rectangle(X-cstWidth, Y1-cstWidth, X+cstWidth, Y1+cstWidth);
  Self.Canvas.Rectangle(XX-cstWidth, Y1-cstWidth, XX+cstWidth, Y1+cstWidth);
end;

procedure TForm1.DrawRectangle(X, Y, Width, Height: Integer);
begin
  Self.Canvas.Polyline([ Point(X, Y), Point(X+Width, Y), Point(X+Width, Y+Height), Point(X, Y+Height), Point(X, Y) ]);
end;



end.

